@@ -1,5 +1,5 @@
 #![deny(missing_docs)]
-#![doc(html_root_url = "https://dabo.guru/rust/throw/throw/")]
+#![doc(html_root_url = "https://docs.rs/throw/0.1.2")]
 //! Throw!
 //! ------
 //!
@@ -74,9 +74,9 @@
 //! #       /*
 //!         panic!("{}", e);
 //! #       */
-//! #       assert_eq!(format!("{}", e), "Error: No such file or directory (os error 2)\
-//! #       \n\tat 16:23 in rust_out (<anon>)\
-//! #       \n\tat 9:19 in rust_out (<anon>)");
+//! #       let err = e.to_string();
+//! #       assert!(err.starts_with("Error: No such file or directory (os error 2)\
+//! #       \n\tat "), "mangled error message: {}", err);
 //!     }
 //! }
 //! ```
@@ -118,8 +118,8 @@
 //! #   /*
 //!     possibly_fails().unwrap()
 //! #   */
-//! #   assert_eq!(format!("{}", possibly_fails().unwrap_err()), "Error: oops\
-//! #   \n\tat 6:8 in rust_out (<anon>)")
+//! #   let err = possibly_fails().unwrap_err().to_string();
+//! #   assert!(err.starts_with("Error: oops\n\tat "), "mangled error message: {}",  err);
 //! }
 //! ```
 //!
@@ -171,8 +171,12 @@ impl ErrorPoint {
     }
 
     #[doc(hidden)]
-    pub fn __construct(line: u32, column: u32, module_path: &'static str, file: &'static str)
-            -> ErrorPoint {
+    pub fn __construct(
+        line: u32,
+        column: u32,
+        module_path: &'static str,
+        file: &'static str,
+    ) -> ErrorPoint {
         ErrorPoint {
             line: line,
             column: column,
@@ -186,7 +190,7 @@ impl ErrorPoint {
 /// which the error was propagated.
 pub struct Error<E> {
     points: Vec<ErrorPoint>,
-    original_error: E,
+    error: E,
 }
 
 impl<E> Error<E> {
@@ -194,7 +198,7 @@ impl<E> Error<E> {
     pub fn new(error: E) -> Error<E> {
         Error {
             points: Vec::new(),
-            original_error: error,
+            error: error,
         }
     }
 
@@ -212,45 +216,83 @@ impl<E> Error<E> {
     }
 
     /// Gets the original error which this Error was constructed with.
+    #[deprecated = "use `error` instead."]
     #[inline]
     pub fn original_error(&self) -> &E {
-        &self.original_error
+        self.error()
     }
-    
+
+    /// Gets the original error which this Error was constructed with.
+    #[inline]
+    pub fn error(&self) -> &E {
+        &self.error
+    }
+
     /// Move the original error out.
     #[inline]
     pub fn into_origin(self) -> E {
-        self.original_error
+        self.into_error()
+    }
+
+    /// Take out the original error and transform into another type
+    /// where the original error can transform into that type.
+    #[inline]
+    pub fn into_error<N>(self) -> N
+    where
+        E: Into<N>,
+    {
+        self.error.into()
     }
 
     /// Transforms this Error<OldError> into Error<NewError>. This isn't implemented as an Into or
     /// From implementation because it would conflict with the blanket implementations in stdlib.
-    pub fn transform<NE>(self) -> Error<NE> where E: Into<NE> {
+    pub fn transform<NE>(self) -> Error<NE>
+    where
+        E: Into<NE>,
+    {
         Error {
             points: self.points,
-            original_error: self.original_error.into(),
+            error: self.error.into(),
         }
     }
 }
 
-impl<E> fmt::Display for Error<E> where E: fmt::Display {
+impl<E> fmt::Display for Error<E>
+where
+    E: fmt::Display,
+{
     fn fmt(&self, fmt: &mut fmt::Formatter) -> std::result::Result<(), fmt::Error> {
-        try!(write!(fmt, "Error: {}", self.original_error));
+        try!(write!(fmt, "Error: {}", self.error));
         for point in self.points.iter().rev() {
-            try!(write!(fmt, "\n\tat {}:{} in {} ({})", point.line(), point.column(),
-                point.module_path(), point.file()));
+            try!(write!(
+                fmt,
+                "\n\tat {}:{} in {} ({})",
+                point.line(),
+                point.column(),
+                point.module_path(),
+                point.file()
+            ));
         }
 
         Ok(())
     }
 }
 
-impl<E> fmt::Debug for Error<E> where E: fmt::Debug {
+impl<E> fmt::Debug for Error<E>
+where
+    E: fmt::Debug,
+{
     fn fmt(&self, fmt: &mut fmt::Formatter) -> std::result::Result<(), fmt::Error> {
-        try!(write!(fmt, "Error: {:?}", self.original_error));
+        try!(write!(fmt, "Error: {:?}", self.error));
         for point in self.points.iter().rev() {
-            try!(write!(fmt, "\n\tat {}:{} in {} ({})", point.line(), point.column(),
-                point.module_path(), point.file()));
+            try!(write!(
+                fmt,
+                "\n\tat {}:{} in {} ({})",
+                point.line(),
+                point.column(),
+                point.module_path(),
+                point.file()
+            ));
         }
 
         Ok(())
@@ -264,19 +306,27 @@ macro_rules! up {
             Ok(v) => v,
             Err(e) => {
                 // re-assignment for a better error message if up!() is used incorrectly
-                let mut e: $crate::Error<_> = e.transform();
-                e.__push_point($crate::ErrorPoint::__construct(
-                    line!(),
-                    column!(),
-                    module_path!(),
-                    file!(),
-                ));
-                return Err(e);
+                return Err(__with_new_errorpoint!(e.transform()));
             },
         }
     );
 }
 
+#[doc(hidden)]
+#[macro_export]
+macro_rules! __with_new_errorpoint {
+    ($e:expr) => ({
+        let mut e = $e;
+        e.__push_point($crate::ErrorPoint::__construct(
+            line!(),
+            column!(),
+            module_path!(),
+            file!(),
+        ));
+        e
+    })
+}
+
 #[macro_export]
 macro_rules! throw {
     ($e:expr) => (
@@ -290,13 +340,6 @@ macro_rules! throw {
 #[macro_export]
 macro_rules! throw_new {
     ($e:expr) => ({
-        let mut e: $crate::Error<_> = $crate::Error::new($e.into());
-        e.__push_point($crate::ErrorPoint::__construct(
-            line!(),
-            column!(),
-            module_path!(),
-            file!(),
-        ));
-        return Err(e);
+        return Err(__with_new_errorpoint!($crate::Error::new($e.into())));
     })
 }
